@@ -6,9 +6,10 @@ use Carp ;
 use IO::File ;
 use IPC::Open3 ;
 use IO::Socket ;
+use Text::ParseWords ;
 use Inline::Java::Portable ;
 
-$Inline::Java::JVM::VERSION = '0.50_90' ;
+$Inline::Java::JVM::VERSION = '0.50_91' ;
 
 my %SIGS = () ;
 
@@ -47,17 +48,21 @@ sub new {
 	if ($o->get_java_config('JNI')){
 		Inline::Java::debug(1, "JNI mode") ;
 
+		# Split args and remove quotes
+		my @args = map {s/(['"])(.*)\1/$2/ ; $_}
+			parse_line('\s+', 1, $args) ;
 		my $jni = new Inline::Java::JNI(
 			$ENV{CLASSPATH} || '',
-			$args || '',
+			\@args,
 			$this->{embedded},
 			Inline::Java::get_DEBUG(),
+			$o->get_java_config('NATIVE_DOUBLES'),
 		) ;
 		$jni->create_ijs() ;
 
 		$this->{JNI} = $jni ;
 	}
-	else{
+	else {
 		Inline::Java::debug(1, "client/server mode") ;
 
 		my $debug = Inline::Java::get_DEBUG() ;
@@ -116,7 +121,8 @@ sub new {
 
 		my $shared = ($this->{shared} ? "true" : "false") ;
 		my $priv = ($this->{private} ? "true" : "false") ;
-		my $cmd = Inline::Java::Portable::portable("SUB_FIX_CMD_QUOTES", "\"$java\" $args org.perl.inline.java.InlineJavaServer $debug $this->{port} $shared $priv") ;
+		my $native_doubles = ($o->get_java_config('NATIVE_DOUBLES') ? "true" : "false") ;
+		my $cmd = Inline::Java::Portable::portable("SUB_FIX_CMD_QUOTES", "\"$java\" $args org.perl.inline.java.InlineJavaServer $debug $this->{port} $shared $priv $native_doubles") ;
 		Inline::Java::debug(2, $cmd) ;
 		if ($o->get_config('UNTAINT')){
 			($cmd) = $cmd =~ /(.*)/ ;
